@@ -12,6 +12,7 @@ from syft.grid.clients.model_centric_fl_client import ModelCentricFLClient
 from datetime import date
 
 import torch as th
+from torch import nn
 
 import os
 from websocket import create_connection
@@ -45,7 +46,7 @@ def get_my_purchased_datasets(password):
         u = Cognito(userPoolId, clientId, username=user_id)
         u.authenticate(password=password)
     except:
-        return {"Error": "Failed to authenticate User"}
+        raise Exception("Error: Failed to authenticate User")
 
     accessId = u.access_token
     # PyGrid masterNode address from constants.py
@@ -56,7 +57,10 @@ def get_my_purchased_datasets(password):
     headers["Authorization"] = "Bearer " + accessId
     headers["Content-Type"] = "application/json"
     node = {"user_id": user_id}
-    resp = requests.post(masterNodeAddy, headers=headers, json=node)
+    try:
+        resp = requests.post(masterNodeAddy, headers=headers, json=node)
+    except:
+        raise Exception("Error: Failed to contact Artificien masternode API. Contact Artificien Support")
     resp = resp.json()
     print(resp)
     return
@@ -214,7 +218,7 @@ def def_avg_plan(model_params, func=None):
     return avg_plan
 
 
-def artificien_connect(dataset_id, model_id, password):
+def artificien_connect(dataset_id, model_id, features, labels, password):
     """ Function to connect to artificien PyGrid node """
     #api authentication using cognito
     count = 0
@@ -222,7 +226,7 @@ def artificien_connect(dataset_id, model_id, password):
         u = Cognito(userPoolId, clientId, username=os.environ['JUPYTERHUB_USER'])
         u.authenticate(password=password)
     except:
-        exit({"Error": "Failed to authenticate User"})
+        raise Exception("Error : Failed to Cognito authenticate Jupyter User. Check your password.")
 
     accessId = u.access_token
     # PyGrid masterNode address from constants.py
@@ -232,18 +236,23 @@ def artificien_connect(dataset_id, model_id, password):
     headers = CaseInsensitiveDict()
     headers["Authorization"] = "Bearer " + accessId
     headers["Content-Type"] = "application/json"
-    node = {"dataset_id": dataset_id, "model_id": model_id}
-    resp = requests.post(masterNodeAddy, headers=headers, json=node)
+    node = {"dataset_id": dataset_id, "model_id": model_id, "features": features, "labels": labels}
+    try:
+        resp = requests.post(masterNodeAddy, headers=headers, json=node)
+    except:
+        raise Exception("Error: Failed to contact Artificien masternode API. Contact Artificien Support")
+
     resp = resp.json()
-    print(resp)
-    #print(resp.get('status'))
     # ping masternode until the ready status is recieved (the node is deployed)
     while resp.get('status') != 'ready':
         count = count + 1
         if 'error' in resp:
-            exit({'error': 'failed to connect'})
+            raise Exception(resp)
         time.sleep(30)
-        resp = requests.post(masterNodeAddy, headers=headers, json=node)
+        try:
+            resp = requests.post(masterNodeAddy, headers=headers, json=node)
+        except:
+            raise Exception("Error: Failed to contact Artificien masternode API. Contact Artificien Support")
         resp = resp.json()
         print(resp.get('status'))
 
@@ -257,31 +266,40 @@ def artificien_connect(dataset_id, model_id, password):
         time.sleep(180)
 
     # connect to grid
-    grid = ModelCentricFLClient(id=dataset_id, address=nodeURL, secure=False)
-    grid.connect()  # These name/version you use in worker
+    try:
+        grid = ModelCentricFLClient(id=dataset_id, address=nodeURL, secure=False)
+        grid.connect()  # These name/version you use in worker
+    except:
+        raise Exception("Error: Failed to connect to the Pygrid Node")
     # return grid
     return grid
 
-def send_model(name, version, batch_size, learning_rate, max_updates, model_params, training_plan, avg_plan, dataset_id, password):
+def send_model(name, version, batch_size, learning_rate, max_updates, model_params, training_plan, avg_plan, dataset_id, features, labels, password):
     """ Function to send model to node """
 
     # Add username to the model name so as to avoid conflicts across users
     name = name + '-' + version + '-' + os.environ['JUPYTERHUB_USER']
     table = dynamodb.Table('model_table')
 
-    table.put_item(
-        Item={
-            'model_id': name,
-            'active_status': 1,
-            'version': version,
-            'dataset': dataset_id,
-            'date_submitted': str(date.today()),
-            'owner_name': str(os.environ['JUPYTERHUB_USER']),
-            'percent_complete': 0,
-        }
-    )
+    try:
+        table.put_item(
+            Item={
+                'model_id': name,
+                'active_status': 1,
+                'version': version,
+                'dataset': dataset_id,
+                'date_submitted': str(date.today()),
+                'owner_name': str(os.environ['JUPYTERHUB_USER']),
+                'percent_complete': 0,
+                'devices_trained_this_cycle': 0,
+                'loss_this_cycle': -1,
+                'acc_this_cycle': -1 # will remain -1 unless accuracy is relevant to the use case
+            }
+        )
+    except:
+        return {"error": "failed to put to dynamodb"}
 
-    grid = artificien_connect(dataset_id, name, password)
+    grid = artificien_connect(dataset_id, name, features, labels, password)
 
     client_config = {
         "name": name,
@@ -301,7 +319,8 @@ def send_model(name, version, batch_size, learning_rate, max_updates, model_para
         "max_diffs": 1,  # number of diffs to collect before avg
         "minimum_upload_speed": 0,
         "minimum_download_speed": 0,
-        "iterative_plan": True  # tells PyGrid that avg plan is executed per diff
+        "iterative_plan": True,  # tells PyGrid that avg plan is executed per diff
+        "model_name": name  # not really supposed to go here, but allows PyGrid to get model name easily
     }
 
     model_params_state = State(
@@ -311,76 +330,27 @@ def send_model(name, version, batch_size, learning_rate, max_updates, model_para
         ]
     )
 
-    response = grid.host_federated_training(
-        model=model_params_state,
-        client_plans={'training_plan': training_plan},
-        client_protocols={},
-        server_averaging_plan=avg_plan,
-        client_config=client_config,
-        server_config=server_config
-    )
+    try:
+        response = grid.host_federated_training(
+            model=model_params_state,
+            client_plans={'training_plan': training_plan},
+            client_protocols={},
+            server_averaging_plan=avg_plan,
+            client_config=client_config,
+            server_config=server_config
+        )
+    except:
+        #node hasn't been hit for a while, let's try again
+        try:
+            response = grid.host_federated_training(
+                model=model_params_state,
+                client_plans={'training_plan': training_plan},
+                client_protocols={},
+                server_averaging_plan=avg_plan,
+                client_config=client_config,
+                server_config=server_config
+            )
+        except:
+            raise Exception("Error: Failed to contact node")
 
-    return print("Host response:", response)
-
-def sendWsMessage(data):
-    """ Helper function to make WS requests """
-
-    ws = create_connection('ws://' + gridAddress)
-
-    ws.send(json.dumps(data))
-    message = ws.recv()
-    return json.loads(message)
-
-
-def check_hosted_model(name, version):
-    
-    cycle_request = {
-        "type": "model-centric/cycle-request",
-        "data": {
-            "worker_id": auth_response['data']['worker_id'],
-            "model": name,
-        
-            "version": version,
-            "ping": 1,
-            "download": 10000,
-            "upload": 10000,
-        }
-    }
-    cycle_response = sendWsMessage(cycle_request)
-    print('Cycle response:', json.dumps(cycle_response, indent=2))
-
-    worker_id = auth_response['data']['worker_id']
-    request_key = cycle_response['data']['request_key']
-    model_id = cycle_response['data']['model_id'] 
-    training_plan_id = cycle_response['data']['plans']['training_plan']
-    
-    # Model
-    req = requests.get(f"http://{gridAddress}/model-centric/get-model?worker_id={worker_id}&request_key={request_key}&model_id={model_id}")
-    model_data = req.content
-    pb = StatePB()
-    pb.ParseFromString(req.content)
-    model_params_downloaded = protobuf.serde._unbufferize(hook.local_worker, pb)
-    print("Params shapes:", [p.shape for p in model_params_downloaded.tensors()])
-    
-    # Plan "list of ops"
-    req = requests.get(f"http://{gridAddress}/model-centric/get-plan?worker_id={worker_id}&request_key={request_key}&plan_id={training_plan_id}&receive_operations_as=list")
-    pb = PlanPB()
-    pb.ParseFromString(req.content)
-    plan_ops = protobuf.serde._unbufferize(hook.local_worker, pb)
-    print(plan_ops.code)
-    print(plan_ops.torchscript)
-    
-    # Plan "torchscript"
-    req = requests.get(f"http://{gridAddress}/model-centric/get-plan?worker_id={worker_id}&request_key={request_key}&plan_id={training_plan_id}&receive_operations_as=torchscript")
-    pb = PlanPB()
-    pb.ParseFromString(req.content)
-    plan_ts = protobuf.serde._unbufferize(hook.local_worker, pb)
-    print(plan_ts.code)
-    print(plan_ts.torchscript.code)
-    
-    # Plan "tfjs"
-    req = requests.get(f"http://{gridAddress}/model-centric/get-plan?worker_id={worker_id}&request_key={request_key}&plan_id={training_plan_id}&receive_operations_as=tfjs")
-    pb = PlanPB()
-    pb.ParseFromString(req.content)
-    plan_tfjs = protobuf.serde._unbufferize(hook.local_worker, pb)
-    print(plan_tfjs.code)
\ No newline at end of file
+    return print("Host response:", response)
\ No newline at end of file
